/*
* Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
*
* Author:     leilong  <leilong@uniontech.com>
*
* Maintainer: leilong  <leilong@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TEST_JOBTYPELISTVIEW_H
#define TEST_JOBTYPELISTVIEW_H

#include "jobtypelistview.h"
#include "../third-party_stub/stub.h"
#include <QObject>
#include <QTest>
#include <gtest/gtest.h>

static QList<JobTypeInfo> jobtypelistview_stub_JobTypeInfos()
{
    QList<JobTypeInfo> infos;
    infos.push_back(JobTypeInfo(1, "1", 0, "#000000"));
    infos.push_back(JobTypeInfo(2, "2", 1, "#000000"));
    infos.push_back(JobTypeInfo(3, "3", 2, "#000000"));

    return infos;
}


class test_jobtypelistview: public::testing::Test
{
public:
    test_jobtypelistview();

    void SetUp()
    {
        Stub stub;
        stub.set(ADDR(JobTypeInfoManager, getJobTypeList), jobtypelistview_stub_JobTypeInfos);
        mWidget = new JobTypeListView();
    }

    void TearDown()
    {
        delete mWidget;
        mWidget = nullptr;
    }
protected:
    JobTypeListView *mWidget = nullptr;
};

#endif // TEST_JOBTYPELISTVIEW_H
