#include <DSettings>

void GenerateSettingTranslate()
{
    auto manage_calendar = QObject::tr("Manage calendar");
    auto event_type = QObject::tr("Event types");
    Q_UNUSED(manage_calendar);
    Q_UNUSED(event_type);
}
