/*
* Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd.
*
* Author:     leilong  <leilong@uniontech.com>
*
* Maintainer: leilong  <leilong@uniontech.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "test_cradiobutton.h"

test_cradiobutton::test_cradiobutton()
{

}

TEST_F(test_cradiobutton, setColor_001)
{
    QColor color("#000000");
    mWidget->setColor(color);
    EXPECT_EQ(mWidget->getColor(), color);
}

TEST_F(test_cradiobutton, paintEvent_001)
{
    QColor color("#000000");
    mWidget->setColor(color);
    mWidget->setChecked(true);
    QPaintEvent *e = new QPaintEvent(QRect());
    mWidget->paintEvent(e);
    delete e;
}

TEST_F(test_cradiobutton, paintEvent_002)
{
    QColor color("#000000");
    mWidget->setColor(color);
    mWidget->setChecked(false);
    QPaintEvent *e = new QPaintEvent(QRect());
    mWidget->paintEvent(e);
    delete e;
}
